/**
 * @file ex2.c
 * @author Florian Burgener
 * @brief Exercice 2
 * @version 1.0
 * @date 2022-01-25
 * 
 * @copyright Copyright (c) 2021
 * 
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#define VALUES_LENGTH 4

// LinkedListNode.h

typedef struct LinkedListNode {
    struct LinkedListNode *next;
    int32_t value;
} LinkedListNode;

/**
 * @brief Creation of a new node.
 * 
 * @param value 
 * @return LinkedListNode* 
 */
LinkedListNode *linked_list_node_new(int32_t value);

/**
 * @brief Deletion of a node.
 * 
 * @param node 
 */
void linked_list_node_delete(LinkedListNode **node);

// LinkedListNode.c

LinkedListNode *linked_list_node_new(int32_t value) {
    LinkedListNode *node = (LinkedListNode *)malloc(sizeof(LinkedListNode));

    node->next = NULL;
    node->value = value;

    return node;
}

void linked_list_node_delete(LinkedListNode **node) {
    free(*node);
    *node = NULL;
}

// LinkedList.h

typedef struct {
    LinkedListNode *root;
} LinkedList;

/**
 * @brief Creation of a new linked list.
 * 
 * @return LinkedList* 
 */
LinkedList *linked_list_new();

/**
 * @brief Deletion of the linked list.
 * 
 * @param list 
 */
void linked_list_delete(LinkedList **list);

/**
 * @brief Count the number of nodes in the list.
 * 
 * @param list 
 * @return int32_t 
 */
int32_t linked_list_count(LinkedList *list);

/**
 * @brief Checks if the list is empty or not.
 * 
 * @param list 
 * @return true 
 * @return false 
 */
bool linked_list_is_empty(LinkedList *list);

/**
 * @brief Adds a value at the end of the list.
 * 
 * @param list 
 * @param value 
 * @return LinkedListNode* 
 */
LinkedListNode *linked_list_add_last(LinkedList *list, int32_t value);

/**
 * @brief Removes a node from the list.
 * 
 * @param list 
 * @param node 
 */
void linked_list_remove(LinkedList *list, LinkedListNode **node);

/**
 * @brief Displays the list.
 * 
 * @param list 
 */
void linked_list_print(LinkedList *list);

/**
 * @brief Inserts a value at the desired index, if the index does not exist does nothing.
 * 
 * @param list 
 * @param index 
 * @param value 
 */
void linked_list_insert_at_index(LinkedList *list, int32_t index, int32_t value);

/**
 * @brief Deletes a node at the desired index, if the index does not exist does nothing.
 * 
 * @param list 
 * @param index 
 */
void linked_list_remove_at_index(LinkedList *list, int32_t index);

// LinkedList.c

LinkedList *linked_list_new() {
    LinkedList *list = (LinkedList *)malloc(sizeof(LinkedList));

    list->root = NULL;

    return list;
}

void linked_list_delete(LinkedList **list) {
    LinkedListNode *current_node = (*list)->root;

    while (current_node != NULL) {
        LinkedListNode *current_node_copy = current_node;
        current_node = current_node->next;
        free(current_node_copy);
    }

    free(*list);
    *list = NULL;
}

int32_t linked_list_count(LinkedList *list) {
    LinkedListNode *current_node = list->root;
    int32_t count = 0;

    while (current_node != NULL) {
        count += 1;
        current_node = current_node->next;
    }

    return count;
}

bool linked_list_is_empty(LinkedList *list) {
    return linked_list_count(list) == 0;
}

LinkedListNode *linked_list_add_last(LinkedList *list, int32_t value) {
    LinkedListNode *new_node = linked_list_node_new(value);

    if (linked_list_is_empty(list)) {
        list->root = new_node;
    } else {
        LinkedListNode *current_node = list->root;

        while (current_node->next != NULL) {
            current_node = current_node->next;
        }

        current_node->next = new_node;
    }

    return new_node;
}

void linked_list_remove(LinkedList *list, LinkedListNode **node) {
    if (list->root == (*node)) {
        list->root = (*node)->next;
    } else {
        LinkedListNode *current_node = list->root;

        while (current_node->next != (*node)) {
            current_node = current_node->next;
        }

        current_node->next = current_node->next->next;
    }

    free(*node);
    *node = NULL;
}

void linked_list_print(LinkedList *list) {
    LinkedListNode *current_node = list->root;

    while (current_node != NULL) {
        printf("%d ", current_node->value);
        current_node = current_node->next;
    }

    printf("\n");
}

void linked_list_insert_at_index(LinkedList *list, int32_t index, int32_t value) {
    if (index == 0) {
        LinkedListNode *new_node = linked_list_node_new(value);
        new_node->next = list->root;
        list->root = new_node;
        return;
    }

    LinkedListNode *current_node = list->root;

    while (current_node != NULL && index != 1) {
        current_node = current_node->next;
        index -= 1;
    }

    if (current_node == NULL) {
        return;
    }

    LinkedListNode *new_node = linked_list_node_new(value);
    new_node->next = current_node->next;
    current_node->next = new_node;
}

void linked_list_remove_at_index(LinkedList *list, int32_t index) {
    LinkedListNode *current_node = list->root;

    while (current_node != NULL && index != 0) {
        current_node = current_node->next;
        index -= 1;
    }

    if (current_node == NULL) {
        return;
    }

    linked_list_remove(list, &current_node);
}

int main() {
    int32_t values_length = VALUES_LENGTH;
    int32_t values[values_length];

    // Reading of the 4 input values to be added to the list.
    for (int32_t i = 0; i < values_length; i += 1) {
        int32_t value;
        scanf("%d", &value);
        values[i] = value;
    }

    // Reading of the index where the value 0 must be inserted.
    int32_t insert_index;
    scanf("%d", &insert_index);

    // Reading of the index where the value must be removed.
    int32_t remove_index;
    scanf("%d", &remove_index);

    printf("\n");

    LinkedList *list = linked_list_new();

    // Addition of the 4 numbers read as input in the list.
    for (int32_t i = values_length - 1; i >= 0; i -= 1) {
        linked_list_add_last(list, values[i]);
    }

    // Insertion of the number 0 at the index specified in input.
    linked_list_insert_at_index(list, insert_index, 0);
    linked_list_print(list);

    // Deletion of the value at the index read in input.
    linked_list_remove_at_index(list, remove_index);
    linked_list_print(list);

    linked_list_delete(&list);
    return EXIT_SUCCESS;
}
