/**
 * @file ex4.c
 * @author Florian Burgener
 * @brief Exercice 4
 * @version 1.0
 * @date 2022-01-25
 * 
 * @copyright Copyright (c) 2021
 * 
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

typedef struct Point {
    int32_t x;
    int32_t y;
} Point;

/**
 * @brief Read the points that are in the input.
 * 
 * @param points_length 
 * @return Point** 
 */
Point **read_points_from_input(int32_t points_length) {
    Point **points = (Point **)malloc(sizeof(Point) * points_length);

    for (int32_t i = 0; i < points_length; i += 1) {
        Point *point = (Point *)malloc(sizeof(Point));
        scanf("%d %d", &point->x, &point->y);
        points[i] = point;
    }

    return points;
}

/**
 * @brief Create a new matrix of size row x column.
 * 
 * @param row_count 
 * @param column_count 
 * @return int32_t** 
 */
int32_t **new_matrix(int32_t row_count, int32_t column_count) {
    int32_t **matrix = (int32_t **)malloc(sizeof(int32_t *) * row_count);

    for (int32_t i = 0; i < row_count; i += 1) {
        matrix[i] = (int32_t *)malloc(sizeof(int32_t) * column_count);
    }

    return matrix;
}

/**
 * @brief Calculate the distance between two points.
 * 
 * @param p1 
 * @param p2 
 * @return int32_t 
 */
int32_t calculate_distance(Point *p1, Point *p2) {
    return abs(p2->x - p1->x) + abs(p2->y - p1->y);
}

/**
 * @brief Calculates all possible distances with the list of points.
 * 
 * @param distances 
 * @param points 
 * @param points_length 
 */
void compute_distances(int32_t **distances, Point **points, int32_t points_length) {
    for (int32_t i = 0; i < points_length; i += 1) {
        for (int32_t j = 0; j < points_length; j += 1) {
            distances[i][j] = calculate_distance(points[i], points[j]);
        }
    }
}

/**
 * @brief Displays the matrix.
 * 
 * @param matrix 
 * @param row_count 
 * @param column_count 
 */
void print_matrix(int32_t **matrix, int32_t row_count, int32_t column_count) {
    for (int32_t i = 0; i < row_count; i += 1) {
        for (int32_t j = 0; j < column_count; j += 1) {
            printf("%d ", matrix[i][j]);
        }

        printf("\n");
    }
}

/**
 * @brief Deletes the matrix from the memory.
 * 
 * @param matrix 
 * @param row_count 
 */
void delete_matrix(int32_t **matrix, int32_t row_count) {
    for (int32_t i = 0; i < row_count; i += 1) {
        free(matrix[i]);
    }

    free(matrix);
}

/**
 * @brief Deletes the points from the memory.
 * 
 * @param points 
 * @param points_length 
 */
void delete_points(Point **points, int32_t points_length) {
    for (int32_t i = 0; i < points_length; i += 1) {
        free(points[i]);
    }

    free(points);
}

int main() {
    int32_t points_length;
    scanf("%d", &points_length);
    Point **points = read_points_from_input(points_length);

    int32_t **distances = new_matrix(points_length, points_length);
    compute_distances(distances, points, points_length);

    printf("\n");
    print_matrix(distances, points_length, points_length);

    delete_matrix(distances, points_length);
    delete_points(points, points_length);

    return EXIT_SUCCESS;
}
